@@ -36,8 +36,9 @@ class CipherHandler:
         except Exception as e:
             raise ValueError(f"Key normalization failed: {e}")
 
-    def _get_numeric_key(self) -> int:
-        return sum(ord(c) for c in self.key)
+    def _get_numeric_key(self) -> str:
+        total_ord_sum = sum(ord(c) for c in self.key)
+        return "".join([str((ord(char) + total_ord_sum + i) % 10) for i, char in enumerate(self.key)])
 
     def _offset(self, index: int) -> int:
         try:
@@ -88,7 +89,8 @@ class CipherHandler:
         if not encrypted_bits or len(encrypted_bits) % 8 != 0:
             raise ValueError("Data biner yang dienkripsi tidak valid atau kosong.")
         decrypted_chars = [
-            chr(int(encrypted_bits[i : i + 8], 2) ^ self.numeric_key % 256) for i in range(0, len(encrypted_bits), 8)
+            chr(int(encrypted_bits[i : i + 8], 2) ^ int(self.numeric_key) % 256)
+            for i in range(0, len(encrypted_bits), 8)
         ]
         return "".join(decrypted_chars)
 
@@ -126,7 +128,7 @@ class CipherHandler:
             raise ValueError(f"Metode enkripsi '{self.method}' tidak dikenali.")
 
     def encrypt_binary(self, plaintext: str) -> str:
-        xor_key = self.numeric_key % 256
+        xor_key = int(self.numeric_key) % 256
         encrypted_bits = "".join(format(ord(char) ^ xor_key, "08b") for char in plaintext)
         return encrypted_bits
 
@@ -141,29 +143,34 @@ class CipherHandler:
         encoded_hex = [hex(ord(text[i]) + ord(self.key[i % len(self.key)])) for i in range(len(text))]
         return self.delimiter.join(encoded_hex)
 
-    def save(self, filename: str, code: str):
+    def save(self, filename: str, code: str, key_by_config: str = None):
         encrypted_code = self.encrypt(code)
         if encrypted_code is None:
             raise ValueError("Encryption failed, cannot save.")
 
         to_hex = lambda s: s.encode("utf-8").hex()
+
+        if key_by_config is not None:
+            key_expression = key_by_config
+        else:
+            key_expression = repr(self.key)
+
         hex_map = {
-            "m": to_hex("nsdev"),
-            "c": to_hex("CipherHandler"),
+            "n": to_hex("nsdev"),
+            "C": to_hex("CipherHandler"),
             "b": to_hex("builtins"),
             "t": to_hex("types"),
-            "gk": to_hex("globals"),
-            "bk": to_hex("__builtins__"),
-            "ik": to_hex("__import__"),
-            "gak": to_hex("getattr"),
-            "ck": to_hex("compile"),
-            "fk": to_hex("FunctionType"),
+            "g": to_hex("globals"),
+            "i": to_hex("__import__"),
+            "a": to_hex("getattr"),
+            "c": to_hex("compile"),
+            "f": to_hex("FunctionType"),
+            "e": to_hex("eval"),
+            "M": to_hex(self.method),
+            "K": to_hex(key_expression),
         }
 
-        result = (
-            f"(lambda d,o,h,x: (lambda g,a,i,p,t:a(a(a(i(x(h['b'])),x(h['t'])),x(h['fk']))(a(a(i(x(h['b'])),x(h['ck']))(a(a(p(x(h['m'])),x(h['c']))(**o),'decrypt')(d),'<string>','exec'),g()),'__call__'))())(lambda:a(i(x(h['b'])),x(h['gk']))(), (lambda z:z(z))(lambda f:lambda *a,**k:f(f,*a,**k))(lambda _,f,*a,**k:a(i(x(h['b'])),x(h['gak']))(f,*a,**k)), (lambda z:z(z))(lambda f:lambda *a,**k:f(f,*a,**k))(lambda _,f,*a,**k:a(a(a(i(x(h['b'])),x(h['gk']))(),x(h['bk'])),x(h['ik']))(f,*a,**k)), (lambda z:z(z))(lambda f:lambda *a,**k:f(f,*a,**k))(lambda _,f,*a,**k:a(a(a(i(x(h['b'])),x(h['gk']))(),x(h['bk'])),x(h['ik']))(f,*a,**k)), a(i(x(h['b'])),x(h['t']))))"
-            f"('{encrypted_code}', dict(method='{self.method}', key={repr(self.key)}), {hex_map}, lambda h:__import__('builtins').bytes.fromhex(h).decode('utf-8'))"
-        )
+        result = f"(lambda d, h, x: (lambda b, i, g, c, t, f, e: f(c(g(g(i(x(h['n'])), x(h['C']))(**{{'method': x(h['M']), 'key': e(x(h['K']))}}), 'decrypt')(d), '<string>', 'exec'), t())())(__import__(x(h['b'])),lambda n: __import__(x(h['b'])).__dict__[x(h['i'])](n),lambda o, n: __import__(x(h['b'])).__dict__[x(h['a'])](o, n),lambda *a: __import__(x(h['b'])).__dict__[x(h['c'])](*a),lambda: __import__(x(h['b'])).__dict__[x(h['g'])](),lambda *a: __import__(x(h['b'])).__dict__[x(h['a'])](__import__(x(h['t'])), x(h['f']))(*a),lambda s: __import__(x(h['b'])).__dict__[x(h['e'])](s)))('{encrypted_code}', {hex_map}, lambda s: bytes.fromhex(s).decode('utf-8'))"
 
         try:
             with open(filename, "w") as file:
@@ -220,28 +227,32 @@ class AsciiManager(__import__("nsdev").AnsiColors):
         except Exception as e:
             raise Exception(f"Decryption failed: {e}")
 
-    def save_data(self, filename: str, code: str):
+    def save_data(self, filename: str, code: str, key_by_config: str = None):
         try:
             encrypted_code = self.encrypt(code)
 
             to_hex = lambda s: s.encode("utf-8").hex()
+
+            if key_by_config is not None:
+                key_expression = key_by_config
+            else:
+                key_expression = repr(self.no_format_key)
+
             hex_map = {
-                "m": to_hex("nsdev"),
-                "c": to_hex("AsciiManager"),
+                "n": to_hex("nsdev"),
+                "A": to_hex("AsciiManager"),
                 "b": to_hex("builtins"),
                 "t": to_hex("types"),
-                "gk": to_hex("globals"),
-                "bk": to_hex("__builtins__"),
-                "ik": to_hex("__import__"),
-                "gak": to_hex("getattr"),
-                "ck": to_hex("compile"),
-                "fk": to_hex("FunctionType"),
+                "g": to_hex("globals"),
+                "i": to_hex("__import__"),
+                "a": to_hex("getattr"),
+                "c": to_hex("compile"),
+                "f": to_hex("FunctionType"),
+                "e": to_hex("eval"),
+                "K": to_hex(key_expression),
             }
 
-            result = (
-                f"(lambda d,k,h,x: (lambda g,a,i,p,t:a(a(a(i(x(h['b'])),x(h['t'])),x(h['fk']))(a(a(i(x(h['b'])),x(h['ck']))(a(a(p(x(h['m'])),x(h['c']))(k),'decrypt')(d),'<string>','exec'),g()),'__call__'))())(lambda:a(i(x(h['b'])),x(h['gk']))(), (lambda z:z(z))(lambda f:lambda *a,**k:f(f,*a,**k))(lambda _,f,*a,**k:a(i(x(h['b'])),x(h['gak']))(f,*a,**k)), (lambda z:z(z))(lambda f:lambda *a,**k:f(f,*a,**k))(lambda _,f,*a,**k:a(a(a(i(x(h['b'])),x(h['gk']))(),x(h['bk'])),x(h['ik']))(f,*a,**k)), (lambda z:z(z))(lambda f:lambda *a,**k:f(f,*a,**k))(lambda _,f,*a,**k:a(a(a(i(x(h['b'])),x(h['gk']))(),x(h['bk'])),x(h['ik']))(f,*a,**k)), a(i(x(h['b'])),x(h['t']))))"
-                f"({str(encrypted_code)}, {repr(self.no_format_key)}, {hex_map}, lambda h:__import__('builtins').bytes.fromhex(h).decode('utf-8'))"
-            )
+            result = f"(lambda d, h, x: (lambda b, i, g, c, t, f, e: f(c(g(g(i(x(h['n'])), x(h['A']))(e(x(h['K']))), 'decrypt')(d), '<string>', 'exec'), t())())(__import__(x(h['b'])),lambda n: __import__(x(h['b'])).__dict__[x(h['i'])](n),lambda o, n: __import__(x(h['b'])).__dict__[x(h['a'])](o, n),lambda *a: __import__(x(h['b'])).__dict__[x(h['c'])](*a),lambda: __import__(x(h['b'])).__dict__[x(h['g'])](),lambda *a: __import__(x(h['b'])).__dict__[x(h['a'])](__import__(x(h['t'])), x(h['f']))(*a),lambda s: __import__(x(h['b'])).__dict__[x(h['e'])](s)))({str(encrypted_code)}, {hex_map}, lambda s: bytes.fromhex(s).decode('utf-8'))"
 
             with open(filename, "w") as file:
                 file.write(result)
